/******************************************************************************
* C Source: Plate List                                                        
*                                                                             
* File:                                                                       
*    platelst.c                                                               
*                                                                             
* Description:                                                                
*    This set of functions performs various manipulations on a linked list    
*    (see lnkd_lst.c and lnkd_lst.h) of Plates (see plate.c and plate.h),     
*    each node for an individual plate. (see platelst.h).                     
*                                                                             
* Module List:                                                                
*    Add_Plate_to_List                                                        
*       Insert a plate into the top of a plate list.                          
*    Create_Empty_Plate_List                                                  
*       Create a new empty plate list.                                        
*    Delete_Plate_List                                                        
*       Delete the linked list for a plate list, but not the plate data the   
*       list points to.                                                       
*    Delete_Plate_List_Data                                                   
*       Delete the linked list for a plate list, AND the plate data the       
*       list points to.                                                       
*    Find_Plate_in_List                                                       
*       Find plate with given ID in plate list.                               
*    Make_Plate_List_File_Name                                                
*       Create full plate list file name                                      
*    Read_Plate_List                                     
*       Read data from plate list on disk.                                    
*    Read_Version_1_Plate_List                                     
*       Read data from version 1 plate list on disk.
*    Read_Version_2_Plate_List                                     
*       Read data from version 2 plate list on disk.
*                                                                             
* History:                                                                    
*    Created    : 15-JUN-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*                                                                             
*    Updated    : 24-OCT-96 by J. Doggett,                                    
*       Redelivered Find_Plate_in_List function.                            
*    Updated    : 21-NOV-96 by J. Doggett,                                    
*       Renamed original Read_Plate_List function to Read_Version_1_Plate_List
*          and also updated it.
*       Created new version of Read_Plate_List function.
*       Created new Read_Version_2_Plate_List function.
*    Updated    : 26-FEB-97 by J. Doggett,                                    
*       Redelivered Make_Plate_List_File_Name function.
*    Redelivered: xx-xxx-xx by J. Doggett,                                    
*       GetImage Version 2.0x.                                                
*                                                                             
* Copyright (c) 1993, 1994, 1997, Association of Universities for Research in 
* Astronomy, Inc.  All Rights Reserved.                                       
* Produced under National Aeronautics and Space Administration grant          
* NAG W-2166.                                                                 
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* access function not prototyped in standard place */
#if defined __VMS

/* *** Plate List Introduction ***                                         2 */


#  include <unixio.h>
#elif defined __UNIX
#  include <unixstd.h>
#else
#endif

#include "platelst.h"

#include "astronmy.h"
#include "config.h"
#include "filenams.h"
#include "generic.h"
#include "lnkd_lst.h"
#include "plate.h"

/******************************************************************************
* Module:                                                                     
*    Add_Plate_to_List                                                        
*                                                                             
* File:                                                                       
*    platelst.c                                                               
*                                                                             
* Description:                                                                
*    Adds a plate to the top of a plate list.                                 
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       List                                                                  
*          PLATE_LIST, Reference                                              
*             The plate list to which we're adding the plate.                 
*       Plate                                                                 
*          PLATE, Reference                                                   
*             The plate we're adding to the plate list.                       
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 15-JUN-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
******************************************************************************/
/*********************************
* Add_Plate_to_List declarations *
*********************************/
void       Add_Plate_to_List( List, Plate )
PLATE_LIST List ;
PLATE      Plate;
{
/******************************************
* Add_Plate_to_List begin executable code *
******************************************/
   Insert_Linked_List_Data( List, Plate );

   return;
}
/****************************************
* Add_Plate_to_List end executable code *
****************************************/

/******************************************************************************
* Module:                                                                     
*    Create_Empty_Plate_List                                                  
*                                                                             
* File:                                                                       
*    platelst.c                                                               
*                                                                             
* Description:                                                                
*    Create a new empty plate list, allocating memory for the header.         
*                                                                             
* Return Value:                                                               
*    PLATE_LIST: The newly created plate.                                     
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       none                                                                  
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 15-JUN-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
******************************************************************************/
/***************************************
* Create_Empty_Plate_List declarations *
***************************************/
PLATE_LIST Create_Empty_Plate_List()
{
/************************************************
* Create_Empty_Plate_List begin executable code *
************************************************/
   return ( PLATE_LIST )Create_Empty_Linked_List();
}
/**********************************************
* Create_Empty_Plate_List end executable code *
**********************************************/

/******************************************************************************
* Module:                                                                     
*    Delete_Plate_List                                                        
*                                                                             
* File:                                                                       
*    platelst.c                                                               
*                                                                             
* Description:                                                                
*    Free memory allocated for plate list, but not the data it points to.     
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       List                                                                  
*          PLATE_LIST, Reference                                              
*             The plate list we're deleting from memory.                      
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 15-JUN-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
******************************************************************************/
/*********************************
* Delete_Plate_List declarations *
*********************************/
void       Delete_Plate_List( List )
PLATE_LIST List;
{
/******************************************
* Delete_Plate_List begin executable code *
******************************************/
   Delete_Linked_List( List );

   return;
}
/****************************************
* Delete_Plate_List end executable code *
****************************************/

/******************************************************************************
* Module:                                                                     
*    Delete_Plate_List_Data                                                   
*                                                                             
* File:                                                                       
*    platelst.c                                                               
*                                                                             
* Description:                                                                
*    Free memory allocated for plate list, but AND the data it points to.     
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       List                                                                  
*          PLATE_LIST, Reference                                              
*             The plate list we're deleting from memory.                      
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 15-JUN-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
******************************************************************************/
/**************************************
* Delete_Plate_List_Data declarations *
**************************************/
void       Delete_Plate_List_Data( List )
PLATE_LIST List;
{
/***********************************************
* Delete_Plate_List_Data begin executable code *
***********************************************/
   Delete_Linked_List_Data( List, Delete_Plate );

   return;
}
/*********************************************
* Delete_Plate_List_Data end executable code *
*********************************************/

/******************************************************************************
* Module:                                                                     
*    Find_Plate_in_List                                                       
*                                                                             
* File:                                                                       
*    platelst.c                                                               
*                                                                             
* Description:                                                                
*    Find plate with given ID and volume label in plate list.  If volume      
*    label is null or zero length, then only the plate id is compared.        
*                                                                             
* Return Value:                                                               
*    PLATE: Plate data found in plate list. NULL is returned if the plate is  
*    not found.                                                               
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       List                                                                  
*          PLATE_LIST, Reference                                              
*             The plate list we're deleting from memory.                      
*       Plate_Id                                                              
*          STRING, Reference                                                  
*             Id of plate to find in the plate list.                          
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 25-OCT-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*                                                                             
*    Updated    : 24-OCT-96 by J. Doggett,                                    
*       Modified to compare volume labels in addition to plate ids.           
*    Redelivered: 24-OCT-96 by J. Doggett,                                    
*       GetImage Version 2.0x.                                                
******************************************************************************/
/**********************************
* Find_Plate_in_List declarations *
**********************************/
PLATE      Find_Plate_in_List( List, Plate_Id, volumeLabel )
PLATE_LIST List       ;
STRING     Plate_Id   ;
STRING     volumeLabel;
{
   struct Plate_Info comparisonPlate;
   PLATE             Plate     ; /* Plate found in plate list */

/*******************************************
* Find_Plate_in_List begin executable code *
*******************************************/
   if ((Plate_Id         == (STRING)NULL) ||
       (strlen(Plate_Id) > 4            )   )
      return (PLATE)NULL;
   strcpy(comparisonPlate.Id         , Plate_Id   );

   if ((volumeLabel        != (STRING)NULL) &&
       (strlen(volumeLabel) > 23          )   )
									      
/* *** Find_Plate_in_List ***                                              2 */


      return (PLATE)NULL;
   if (volumeLabel == (STRING)NULL)
      strcpy(comparisonPlate.volumeLabel, "");
   else
      strcpy(comparisonPlate.volumeLabel, volumeLabel);

   return 
      ( PLATE )Find_Linked_List_Data( List, &comparisonPlate, Compare_Plates);
}
/*****************************************
* Find_Plate_in_List end executable code *
*****************************************/

/******************************************************************************
* Module:                                                                     
*    Make_Plate_List_File_Name                                                
*                                                                             
* File:                                                                       
*    platelst.c                                                               
*                                                                             
* Description:                                                                
*    Create full file name for plate list from given directory, and the       
*    symbolic constants PLATE_LIST_FILE or DEFAULT_PLATE_LIST_FILE.           
*                                                                             
* Return Value:                                                               
*    STRING: Name of plate list file.                                         
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       Directory                                                             
*          STRING, Reference                                                  
*             Name of directory where plate list file resides.                
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 19-NOV-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*
*    Updated    : 26-FEB-97 by J. Doggett,                                    
*       Modified to check translated file name against "" rather than NULL;
*    Redelivered: 26-FEB-97 by J. Doggett,                                    
*       GetImage Version 2.0x.
******************************************************************************/
/*****************************************
* Make_Plate_List_File_Name declarations *
*****************************************/
STRING Make_Plate_List_File_Name(Directory)
STRING Directory;
{
   STRING Plate_List_File_Name     ; /* Place for just the file name */
   STRING Full_Plate_List_File_Name; /* File name with path          */

/**************************************************
* Make_Plate_List_File_Name begin executable code *
**************************************************/
   Plate_List_File_Name = Translate_Environment_Variable( PLATE_LIST_FILE );

   if ( strcmp(Plate_List_File_Name, "") == 0)
      Plate_List_File_Name = DEFAULT_PLATE_LIST_FILE;

   if (!Set_FileNames_Case(Directory, Plate_List_File_Name))
   {
      fprintf(stderr,"\nCould not find copyright file in header directory!\n");
      fprintf(stderr, "Aborting GetImage!!!\n\n");
      exit(EXIT_FAILURE);
   }

   Full_Plate_List_File_Name =

/* *** Make_Plate_List_File_Name ***                                       2 */


         Build_Full_File_Name( Directory, "", Plate_List_File_Name );

   /*
   free(Plate_List_File_Name);

     Plate_List_File_Name =
     Set_FileNames_Version(Directory, Full_Plate_List_File_Name);

   if (Plate_List_File_Name != Full_Plate_List_File_Name)
      if (Plate_List_File_Name != NULL)
         free(Full_Plate_List_File_Name);
      else {
         fprintf(stderr,
               "\nCould not find copyright file in header directory!\n");
         fprintf(stderr, "Aborting GetImage!!!\n\n");
         exit(EXIT_FAILURE);
      }

      Full_Plate_List_File_Name = Plate_List_File_Name;
   */
   if (access(Full_Plate_List_File_Name, 0) != 0 )
   {
      Plate_List_File_Name =
            Translate_Environment_Variable( PLATE_LIST_FILE2 );

      if ( Plate_List_File_Name == NULL )
            Plate_List_File_Name = DEFAULT_PLATE_LIST_FILE;

      if (!Set_FileNames_Case(Directory, Plate_List_File_Name))
      {
         fprintf(stderr,
               "\nCould not find copyright file in header directory!\n");
         fprintf(stderr, "Aborting GetImage!!!\n\n");
         exit(EXIT_FAILURE);
      }

      Full_Plate_List_File_Name =
         Build_Full_File_Name( Directory, "", Plate_List_File_Name );

      free(Plate_List_File_Name);

      Plate_List_File_Name =
            Set_FileNames_Version(Directory, Full_Plate_List_File_Name);

      if (Plate_List_File_Name != Full_Plate_List_File_Name)
         if (Plate_List_File_Name != NULL)
            free(Full_Plate_List_File_Name);
         else {
            fprintf(stderr,
                  "\nCould not find copyright file in header directory!\n");
            fprintf(stderr, "Aborting GetImage!!!\n\n");
            exit(EXIT_FAILURE);
         }

      Full_Plate_List_File_Name = Plate_List_File_Name;
   }

   return Full_Plate_List_File_Name;
}
/************************************************
* Make_Plate_List_File_Name end executable code *
************************************************/

/******************************************************************************
* Module:                                                                     
*    Read_Plate_List                                              
*                                                                             
* File:                                                                       
*    platelst.c                                                               
*                                                                             
* Description:                                                                
*    Read data for plates in plate list on disk into internal plate list.     
*    Originally, this routine expected a single specific format for the
*    plate list file.  This routine now expects plate list files in any
*    one of several specific formats at identified with in the file.
*    The original version of this routine is now called
*    Read_Version_1_Plate_List.
*                                                                             
* Return Value:                                                               
*    PLATE_LIST: New internal plate list created from plate list on disk.     
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       Directory                                                             
*          STRING, Reference                                                  
*             Name of the directory where the plate list file resides.        
*       Plate_File                                                            
*             Name of the plate list file in Directory                        
*          STRING, Reference                                                  
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 21-NOV-96 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 21-NOV-96 by J. Doggett,  
*       GetImage Version 2.0x.              
******************************************************************************/
/*******************************
* Read_Plate_List declarations *
*******************************/
PLATE_LIST Read_Plate_List( Plate_List_File_Name )
STRING     Plate_List_File_Name;
{
   FILE * Plate_List_File     ;
   char   firstWord       [81];
   char   plateListVersion[81];
   char   Temp_Char           ;

/****************************************
* Read_Plate_List begin exectuable code *
****************************************/
   printf( "\nReading plate list: %s...\n", Plate_List_File_Name );

/* Find plate list version record */
   Plate_List_File = fopen( Plate_List_File_Name, "r" );
   if ( Plate_List_File == NULL )
   {
      fprintf( stderr, "\nCould not open plate list file %s.\n\n",
                                                        Plate_List_File_Name );
									      
/* *** Read_Plate_List ***                                                 2 */


      exit( EXIT_FAILURE );
   }

   firstWord[0] = PLATE_RECORD_COMMENT_CHARACTER;
   while ( firstWord[0] == PLATE_RECORD_COMMENT_CHARACTER)
   {
      firstWord[0] = getc(Plate_List_File);
      if (firstWord[0] == PLATE_RECORD_COMMENT_CHARACTER)
         while (( (Temp_Char=getc(Plate_List_File)) != PLATE_RECORD_EOL_CHAR)
                            &&
                (Temp_Char                          != EOF                )
                  ) ;
      else
      {
         ungetc(firstWord[0], Plate_List_File);
         switch(fscanf(Plate_List_File, "%s", firstWord))
         {
            case 1:
               break;
            default:
               fprintf(stderr,"Error scanning for plate list version.\n");
	       fclose(Plate_List_File);
               return (PLATE_LIST)NULL;
         }
      }
   }

/* Call function appropriate function to read the plate list file. */
   if (strcmp(firstWord, "^Version") != 0)
   {
      fclose(Plate_List_File);
      return Read_Version_1_Plate_List( Plate_List_File_Name );
   }
   else 
   {
      switch(fscanf(Plate_List_File, "%s", plateListVersion))
      {
         case 1:
            break;
	 default:
	    fprintf(stderr,"Error reading plate list version.\n");
	    fclose(Plate_List_File);
	    return (PLATE_LIST)NULL;
      }
      if (strcmp(plateListVersion, "2") == 0)
      {
	 fclose(Plate_List_File);
	 return Read_Version_2_Plate_List( Plate_List_File_Name );
      }
      else
      {
	 fprintf(stderr,"Unrecognized plate list version.\n");
	 fclose(Plate_List_File);
	 return (PLATE_LIST)NULL;
      }
   }
/**************************************
* Read_Plate_List end exectuable code *
**************************************/
}

/******************************************************************************
* Module:                                                                     
*    Read_Version_1_Plate_List                                              
*                                                                             
* File:                                                                       
*    platelst.c                                                               
*                                                                             
* Description:                                                                
*    Read data for plates in plate list on disk into internal plate list.  The
*    format of the plate list is expected to be "Version 1" which is the
*    version required by GetImage 1.2, 1.3 and 1.3a.
*                                                                             
* Return Value:                                                               
*    PLATE_LIST: New internal plate list created from plate list on disk.     
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       Directory                                                             
*          STRING, Reference                                                  
*             Name of the directory where the plate list file resides.        
*       Plate_File                                                            
*             Name of the plate list file in Directory                        
*          STRING, Reference                                                  
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 15-JUN-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*                                           
*    Updated    : 21-NOV-96 by J. Doggett,  
*       Renamed to Read_Version_1_Plate_List and move "standby" message to
*       new version of Read_Plate_List.
*    Redelivered: 21-NOV-96 by J. Doggett,  
*       GetImage Version 2.0x.              
******************************************************************************/
/*****************************************
* Read_Version_1_Plate_List declarations *
*****************************************/
PLATE_LIST Read_Version_1_Plate_List( Plate_List_File_Name )
STRING     Plate_List_File_Name;
{
   FILE       *Plate_List_File; /* Pointer to plate list file    */
   PLATE_LIST  Plate_List     ; /* New plate list read from disk */
   PLATE Plate     ; /* Place to store plate data read from file */

/**************************************************
* Read_Version_1_Plate_List begin exectuable code *
**************************************************/
   Plate_List_File = fopen( Plate_List_File_Name, "r" );
   if ( Plate_List_File == NULL )
   {
      fprintf( stderr, "\nCould not open plate list file %s.\n\n",
                                                        Plate_List_File_Name );
      exit( EXIT_FAILURE );
   }

/* *** Read_Version_1_Plate_List ***                                       2 */


/* Read plates into plate list until the file is exhausted */
   Plate_List = Create_Empty_Plate_List();

   Plate = Read_Version_1_Plate_Info(Plate_List_File);
   while ((int)Plate != EOF )
   {
      if ((int)Plate == (int)NULL)
      {
         fprintf(stderr, "\nError reading plate list file %s.\n",
                 Plate_List_File_Name);
         fprintf(stderr, "Aborting...\n\n");
         exit( EXIT_FAILURE );
      }

      Add_Plate_to_List( Plate_List, Plate );

      Plate = Read_Version_1_Plate_Info(Plate_List_File);
   }

   fclose( Plate_List_File );

   return Plate_List;
}
/************************************************
* Read_Version_1_Plate_List end exectuable code *
************************************************/

/******************************************************************************
* Module:                                                                     
*    Read_Version_2_Plate_List                                              
*                                                                             
* File:                                                                       
*    platelst.c                                                               
*                                                                             
* Description:                                                                
*    Read data for plates in plate list on disk into internal plate list.  The
*    format of the plate list is expected to be "Version 2" which is the
*    version accepted by GetImage 2.0 and later.  Version 2 of the plate list
*    file contains astronomical survey, bandpass and digitized survey 
*    formation for each plate.  It also contains default choices for
*    astronomical survey and default bandpass and digitized survey for each
*    astronomical for support of future multi-survey functionality in
*    GetImage.
*
* Return Value:                                                               
*    PLATE_LIST: New internal plate list created from plate list on disk.     
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       Directory                                                             
*          STRING, Reference                                                  
*             Name of the directory where the plate list file resides.        
*       Plate_File                                                            
*             Name of the plate list file in Directory                        
*          STRING, Reference                                                  
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 21-NOV-96 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 21-NOV-96 by J. Doggett,  
*       GetImage Version 2.0x.              
******************************************************************************/
/*****************************************
* Read_Version_2_Plate_List declarations *
*****************************************/
PLATE_LIST Read_Version_2_Plate_List( Plate_List_File_Name )
STRING     Plate_List_File_Name;
{
   FILE       *Plate_List_File; /* Pointer to plate list file    */
   PLATE_LIST  Plate_List     ; /* New plate list read from disk */
   PLATE Plate     ; /* Place to store plate data read from file */

   char   firstWord       [81];
   char   plateListVersion[81];
   char   Temp_Char           ;

/**************************************************
* Read_Version_2_Plate_List begin exectuable code *
**************************************************/
   Plate_List_File = fopen( Plate_List_File_Name, "r" );
   if ( Plate_List_File == NULL )
   {
      fprintf( stderr, "\nCould not open plate list file %s.\n\n",

/* *** Read_Version_2_Plate_List ***                                       2 */

                                                        Plate_List_File_Name );
      exit( EXIT_FAILURE );
   }

/* Find plate list version record */
   Plate_List_File = fopen( Plate_List_File_Name, "r" );
   if ( Plate_List_File == NULL )
   {
      fprintf( stderr, "\nCould not open plate list file %s.\n\n",
                                                        Plate_List_File_Name );
      exit( EXIT_FAILURE );
   }

   firstWord[0] = PLATE_RECORD_COMMENT_CHARACTER;
   while ( firstWord[0] == PLATE_RECORD_COMMENT_CHARACTER)
   {
      firstWord[0] = getc(Plate_List_File);
      if (firstWord[0] == PLATE_RECORD_COMMENT_CHARACTER)
         while (( (Temp_Char=getc(Plate_List_File)) != PLATE_RECORD_EOL_CHAR)
                            &&
                (Temp_Char                          != EOF                )
                  ) ;
      else
      {
         ungetc(firstWord[0], Plate_List_File);
         switch(fscanf(Plate_List_File, "%s", firstWord))
         {
            case 1:
               break;
            default:
               fprintf(stderr,"Error scanning for plate list version.\n");
	       fclose(Plate_List_File);
               return (PLATE_LIST)NULL;
         }
      }
   }

/* Check that plate list is Version 2. */
   if (strcmp(firstWord, "^Version") != 0)
   {
      fprintf(stderr, "Plate list is not Version 2.\n");
      fclose(Plate_List_File);
      return (PLATE_LIST)NULL;
   }

   switch(fscanf(Plate_List_File, "%s", plateListVersion))
   {
      case 1:
         break;
      default:
	 fprintf(stderr,"Error reading plate list version.\n");
	 fclose(Plate_List_File);
	 return (PLATE_LIST)NULL;
   }
   if (strcmp(plateListVersion, "2") != 0)
   {
      fprintf(stderr, "Plate list is not Version 2.\n");
      fclose(Plate_List_File);
      return (PLATE_LIST)NULL;
   }

/* Skip over "defaults" */

/* *** Read_Version_2_Plate_List ***                                       3 */


   do
   {
      while (( (Temp_Char=getc(Plate_List_File)) != PLATE_RECORD_EOL_CHAR)
	                      &&
   	     (Temp_Char                    != EOF )
	     ) ;
      if (Temp_Char == EOF)
      {
         fprintf(stderr, "Premature EOF in plate list.\n");
         fclose(Plate_List_File);
         return (PLATE_LIST)NULL;
      }

      switch(fscanf(Plate_List_File, "%s", firstWord))
      {
         case 1:
            break;
         default:
	    fprintf(stderr,"Error skipping defaults in plate list.\n");
	    fclose(Plate_List_File);
	    return (PLATE_LIST)NULL;
      }
   } while (strcmp(firstWord, "^^") != 0); 

   while (( (Temp_Char=getc(Plate_List_File)) != PLATE_RECORD_EOL_CHAR)
                    &&
          (Temp_Char                    != EOF )
	  ) ;
   if (Temp_Char == EOF)
   {
      fprintf(stderr, "Premature EOF in plate list.\n");
      fclose(Plate_List_File);
      return (PLATE_LIST)NULL;
   }

/* Read plates into plate list until the file is exhausted */
   Plate_List = Create_Empty_Plate_List();

   Plate = Read_Version_2_Plate_Info(Plate_List_File);
   while ((int)Plate != EOF )
   {
      if ((int)Plate == (int)NULL)
      {
         fprintf(stderr, "\nError reading plate list file %s.\n",
                 Plate_List_File_Name);
         fprintf(stderr, "Aborting...\n\n");
         exit( EXIT_FAILURE );
      }

      Add_Plate_to_List( Plate_List, Plate );

      Plate = Read_Version_2_Plate_Info(Plate_List_File);
   }

   fclose( Plate_List_File );

   return Plate_List;
}
/************************************************
* Read_Version_2_Plate_List end exectuable code *
************************************************/
